
clear;
clc;


%% Parameterisation / Calibration / Functional Forms

% parameters

delta=1/5;  % long-term bond equivalent to 5-year bond (inverse shown in paper)
beta=0.96;  % discount factor
gamma=5;    % coefficient of relative risk aversion
Rb=1.02;    % gross risk-free rate on bonds
sigmaU=0.16;    % strd deviation of log of U
sigmaN=0.12;    % strd deviation of log of N
G=1.03;     % drift in permanent component of income
G_ret=1;        % drift in permanent component of retirement income
Rqexcess=0.006; % premium of long-term bond over safe short-term bond
phi=0.035;      % fixed participation cost
psi_s=0.015;    % variable cost of stock holdings
J=54;       % periods lived before death
J_ret=36;       % periods lived before retirement
ret=0.6;    % replacement ratio


% utility function

if gamma==1                              
    util = @(c) log(c);
else
    util = @(c) 1/(1-gamma)*c.^(1-gamma);
end


%% Exogenous Processes

nU=2; % number of states for transitory income shock;
ProbU=[0.5 0.5; 0.5 0.5];
logU=[-0.5*sigmaU^2-sigmaU; -0.5*sigmaU^2+sigmaU];
U=exp(logU);

nN=2; % number of states for permanent income shock;
ProbN=[0.5 0.5; 0.5 0.5];
logN=[-0.5*sigmaN^2-sigmaN; -0.5*sigmaN^2+sigmaN];
N=exp(logN);

nRq=2; %number of states for return paid on long-term bond
aRq=Rb+Rqexcess; sigmaRq=0.008; rhoRq=0.49;
[Rq,ProbRq]=addacooper(nRq,aRq,rhoRq,sigmaRq);

Rs=1+[-0.0917;  0.2183];   % Stock return
nRs = length(Rs); % number of possible states for stock return
ProbRs=[0.5 0.5; 0.5 0.5];


%% Grid

b_min=0;
b_max=4;
b_n=44;
b_grid=(b_min:(b_max-b_min)/(b_n-1):b_max)';

q_min=0;
q_max=5;
q_n=55;
q_grid=(q_min:(q_max-q_min)/(q_n-1):q_max)';

s_min=0;
s_max=7;
s_n=77;
s_grid=(s_min:(s_max-s_min)/(s_n-1):s_max)';

x_min=ret*min(U);
x_max=(max(Rs)*s_max+Rb*b_max+delta*max(Rq)*q_max)/(G*min(N))+max(U)+0.1;
x_n=174;
x_grid=(x_min:(x_max-x_min)/(x_n-1):x_max)';

z_min=q_min; % place z-grid onto q-grid so that constraint q'>=z can hold precisely with equality in case of no participation
z_max=q_max;
z_n=q_n;
z_grid=q_grid;


% Setting up variables

bprime_grid_bqsvec=repelem(b_grid,q_n*s_n);
qprime_grid_bqsvec=repmat(repelem(q_grid,s_n),b_n,1);
sprime_grid_bqsvec=repmat(s_grid,b_n*q_n,1);

xprime_p_Rqprimelow=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_p_Rqprimehigh=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimelow=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimehigh=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimelow=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimehigh=zeros(b_n*q_n*s_n,nN*nU*nRs);

xprime_p_Rqprimelow_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_p_Rqprimehigh_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimelow_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimehigh_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimelow_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimehigh_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);

xprime_p_Rqprimelow_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_p_Rqprimehigh_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimelow_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimehigh_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimelow_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimehigh_ret=zeros(b_n*q_n*s_n,nN*nU*nRs);

xprime_p_Rqprimelow_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_p_Rqprimehigh_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimelow_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
xprime_n_Rqprimehigh_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimelow_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);
zprime_Rqprimehigh_ret_coord=zeros(b_n*q_n*s_n,nN*nU*nRs);


for bqs=1:b_n*q_n*s_n
   
    b=bprime_grid_bqsvec(bqs);
    q=qprime_grid_bqsvec(bqs);
    s=sprime_grid_bqsvec(bqs);
    
    ind=0;
    
        
    for nn=1:nN
        for uu=1:nU
            for rsrs=1:nRs
                ind=ind+1;
                    
                xprime_p_Rqprimelow(bqs,ind)=(Rs(rsrs)*s+Rb*b+delta*Rq(1)*q)/(G*N(nn))+U(uu);
                xprime_p_Rqprimehigh(bqs,ind)=(Rs(rsrs)*s+Rb*b+delta*Rq(2)*q)/(G*N(nn))+U(uu);
                xprime_n_Rqprimelow(bqs,ind)=(Rb*b+delta*Rq(1)*q)/(G*N(nn))+U(uu);
                xprime_n_Rqprimehigh(bqs,ind)=(Rb*b+delta*Rq(2)*q)/(G*N(nn))+U(uu);
                zprime_Rqprimelow(bqs,ind)=(1-delta)*Rq(1)*q/(G*N(nn));
                zprime_Rqprimehigh(bqs,ind)=(1-delta)*Rq(2)*q/(G*N(nn));
                
                xprime_p_Rqprimelow_coord(bqs,ind)=find(x_grid<=xprime_p_Rqprimelow(bqs,ind),1,'last');
                xprime_p_Rqprimehigh_coord(bqs,ind)=find(x_grid<=xprime_p_Rqprimehigh(bqs,ind),1,'last');
                xprime_n_Rqprimelow_coord(bqs,ind)=find(x_grid<=xprime_n_Rqprimelow(bqs,ind),1,'last');
                xprime_n_Rqprimehigh_coord(bqs,ind)=find(x_grid<=xprime_n_Rqprimehigh(bqs,ind),1,'last');               
                zprime_Rqprimelow_coord(bqs,ind)=find(z_grid<=zprime_Rqprimelow(bqs,ind),1,'last');
                zprime_Rqprimehigh_coord(bqs,ind)=find(z_grid<=zprime_Rqprimehigh(bqs,ind),1,'last');   

                xprime_p_Rqprimelow_ret(bqs,ind)=(Rs(rsrs)*s+Rb*b+delta*Rq(1)*q)/(G_ret*mean(N))+ret*mean(U);
                xprime_p_Rqprimehigh_ret(bqs,ind)=(Rs(rsrs)*s+Rb*b+delta*Rq(2)*q)/(G_ret*mean(N))+ret*mean(U);
                xprime_n_Rqprimelow_ret(bqs,ind)=(Rb*b+delta*Rq(1)*q)/(G_ret*mean(N))+ret*mean(U); 
                xprime_n_Rqprimehigh_ret(bqs,ind)=(Rb*b+delta*Rq(2)*q)/(G_ret*mean(N))+ret*mean(U);
                zprime_Rqprimelow_ret(bqs,ind)=(1-delta)*Rq(1)*q/(G_ret*mean(N));
                zprime_Rqprimehigh_ret(bqs,ind)=(1-delta)*Rq(2)*q/(G_ret*mean(N));
                
                xprime_p_Rqprimelow_ret_coord(bqs,ind)=find(x_grid<=xprime_p_Rqprimelow_ret(bqs,ind),1,'last');
                xprime_p_Rqprimehigh_ret_coord(bqs,ind)=find(x_grid<=xprime_p_Rqprimehigh_ret(bqs,ind),1,'last');
                xprime_n_Rqprimelow_ret_coord(bqs,ind)=find(x_grid<=xprime_n_Rqprimelow_ret(bqs,ind),1,'last');
                xprime_n_Rqprimehigh_ret_coord(bqs,ind)=find(x_grid<=xprime_n_Rqprimehigh_ret(bqs,ind),1,'last');               
                zprime_Rqprimelow_ret_coord(bqs,ind)=find(z_grid<=zprime_Rqprimelow_ret(bqs,ind),1,'last');
                zprime_Rqprimehigh_ret_coord(bqs,ind)=find(z_grid<=zprime_Rqprimehigh_ret(bqs,ind),1,'last'); 
                                 
            end   
        end   
    end
        
end

xprime_p_Rqprimelow=reshape(xprime_p_Rqprimelow,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_p_Rqprimehigh=reshape(xprime_p_Rqprimehigh,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimelow=reshape(xprime_n_Rqprimelow,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimehigh=reshape(xprime_n_Rqprimehigh,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimelow=reshape(zprime_Rqprimelow,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimehigh=reshape(zprime_Rqprimehigh,b_n*q_n*s_n*nN*nU*nRs,1);

xprime_p_Rqprimelow_coord=reshape(xprime_p_Rqprimelow_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_p_Rqprimehigh_coord=reshape(xprime_p_Rqprimehigh_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimelow_coord=reshape(xprime_n_Rqprimelow_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimehigh_coord=reshape(xprime_n_Rqprimehigh_coord,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimelow_coord=reshape(zprime_Rqprimelow_coord,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimehigh_coord=reshape(zprime_Rqprimehigh_coord,b_n*q_n*s_n*nN*nU*nRs,1);

xprime_p_Rqprimelow_ret=reshape(xprime_p_Rqprimelow_ret,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_p_Rqprimehigh_ret=reshape(xprime_p_Rqprimehigh_ret,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimelow_ret=reshape(xprime_n_Rqprimelow_ret,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimehigh_ret=reshape(xprime_n_Rqprimehigh_ret,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimelow_ret=reshape(zprime_Rqprimelow_ret,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimehigh_ret=reshape(zprime_Rqprimehigh_ret,b_n*q_n*s_n*nN*nU*nRs,1);

xprime_p_Rqprimelow_ret_coord=reshape(xprime_p_Rqprimelow_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_p_Rqprimehigh_ret_coord=reshape(xprime_p_Rqprimehigh_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimelow_ret_coord=reshape(xprime_n_Rqprimelow_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);
xprime_n_Rqprimehigh_ret_coord=reshape(xprime_n_Rqprimehigh_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimelow_ret_coord=reshape(zprime_Rqprimelow_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);
zprime_Rqprimehigh_ret_coord=reshape(zprime_Rqprimehigh_ret_coord,b_n*q_n*s_n*nN*nU*nRs,1);

prob_bqsind=ones(nN*nU*nRs,1)*ProbN(1,1)*ProbU(1,1)*ProbRs(1,1);


%% Recursion

V_p=zeros(nRq*x_n*z_n,J-1);
bprime_p=zeros(nRq*x_n*z_n,J-1);
qprime_p=zeros(nRq*x_n*z_n,J-1);
sprime_p=zeros(nRq*x_n*z_n,J-1);

V_n=zeros(nRq*x_n*z_n,J-1);
bprime_n=zeros(nRq*x_n*z_n,J-1);
qprime_n=zeros(nRq*x_n*z_n,J-1);

V=zeros(nRq*x_n*z_n,J);
bprime=zeros(nRq*x_n*z_n,J);
qprime=zeros(nRq*x_n*z_n,J);
sprime=zeros(nRq*x_n*z_n,J);
part=zeros(nRq*x_n*z_n,J);

EtVprime_p_bqsvec=zeros(b_n*q_n*s_n,1);
EtVprime_n_bqsvec=zeros(b_n*q_n*s_n,1);


    % Last Period
    
       % definitely non-participant
        bprime(:,J)=0;
        qprime(:,J)=repmat(z_grid,nRq*x_n,1);
        sprime(:,J)=0;
        c=repmat(repelem(x_grid,z_n),nRq,1);
        V(:,J)=util(c);
        
  
    % Retirement
    tic
for jj=J-1:-1:J_ret+1
   jj
     
       for rr=1:nRq
           
           
           % Rqprime low
           
           Vprime_Rqprimelow=reshape(V(1:x_n*z_n,jj+1),z_n,x_n)';
           
           Vprime_p_Rqprimelow=interp2_lin(xprime_p_Rqprimelow_ret,zprime_Rqprimelow_ret,xprime_p_Rqprimelow_ret_coord,zprime_Rqprimelow_ret_coord,x_grid,z_grid,Vprime_Rqprimelow);
           Vprime_p_Rqprimelow=reshape(Vprime_p_Rqprimelow,b_n*q_n*s_n,nN*nU*nRs);
           EtVprime_p_Rqprimelow=Vprime_p_Rqprimelow*prob_bqsind;
           clearvars Vprime_p_Rqprimelow
           
           Vprime_n_Rqprimelow=interp2_lin(xprime_n_Rqprimelow_ret,zprime_Rqprimelow_ret,xprime_n_Rqprimelow_ret_coord,zprime_Rqprimelow_ret_coord,x_grid,z_grid,Vprime_Rqprimelow);
           Vprime_n_Rqprimelow=reshape(Vprime_n_Rqprimelow,b_n*q_n*s_n,nN*nU*nRs);
           EtVprime_n_Rqprimelow=Vprime_n_Rqprimelow*prob_bqsind;
           clearvars Vprime_n_Rqprimelow Vprime_Rqprimelow
           
           
           % Rqprime high 
            
           Vprime_Rqprimehigh=reshape(V(x_n*z_n+1:end,jj+1),z_n,x_n)';
           
           Vprime_p_Rqprimehigh=interp2_lin(xprime_p_Rqprimehigh_ret,zprime_Rqprimehigh_ret,xprime_p_Rqprimehigh_ret_coord,zprime_Rqprimehigh_ret_coord,x_grid,z_grid,Vprime_Rqprimehigh);
           Vprime_p_Rqprimehigh=reshape(Vprime_p_Rqprimehigh,b_n*q_n*s_n,nN*nU*nRs);
           EtVprime_p_Rqprimehigh=Vprime_p_Rqprimehigh*prob_bqsind;
           clearvars Vprime_p_Rqprimehigh
           
           Vprime_n_Rqprimehigh=interp2_lin(xprime_n_Rqprimehigh_ret,zprime_Rqprimehigh_ret,xprime_n_Rqprimehigh_ret_coord,zprime_Rqprimehigh_ret_coord,x_grid,z_grid,Vprime_Rqprimehigh);
           Vprime_n_Rqprimehigh=reshape(Vprime_n_Rqprimehigh,b_n*q_n*s_n,nN*nU*nRs);           
           EtVprime_n_Rqprimehigh=Vprime_n_Rqprimehigh*prob_bqsind;
           clearvars Vprime_n_Rqprimehigh Vprime_Rqprimehigh  
           
           
           EtVprime_p_bqsvec=ProbRq(rr,1).*EtVprime_p_Rqprimelow+ProbRq(rr,2).*EtVprime_p_Rqprimehigh;
           EtVprime_n_bqsvec=ProbRq(rr,1).*EtVprime_n_Rqprimelow+ProbRq(rr,2).*EtVprime_n_Rqprimehigh;
           
           for xx=1:x_n
                for zz=1:z_n
                    
                    %participating
                    c=x_grid(xx)+z_grid(zz)-phi-(bprime_grid_bqsvec+qprime_grid_bqsvec+sprime_grid_bqsvec*(1+psi_s));
                    constr_c=find(c<=0);
                    obj=util(c)+beta*EtVprime_p_bqsvec;
                    obj(constr_c)=-10000000000;
                    constr_q=(repelem(0:b_n-1,(zz-1)*s_n))'*s_n*q_n+repmat((1:(zz-1)*s_n)',b_n,1);
                    obj(constr_q)=-10000000001;
                    
                    [~,max_coordinate]=max(obj);
                    b_coord=floor((max_coordinate-1)/(q_n*s_n))+1;
                    q_coord=rem(floor((max_coordinate-1)/(s_n)),q_n)+1;
                    s_coord=rem(max_coordinate-1,s_n)+1;
                    bprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=b_grid(b_coord);                    
                    qprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=q_grid(q_coord);
                    sprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=s_grid(s_coord);
                    
                    V_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=obj(max_coordinate);
                   
                          
                    %not participating
                    c=x_grid(xx)-bprime_grid_bqsvec;
                    constr_c=find(c<=0);
                    obj=util(c)+beta*EtVprime_n_bqsvec;
                    obj(constr_c)=-10000000000;
                    constr_q=(1:b_n*q_n*s_n)';
                    constr_q((repelem(0:b_n-1,s_n))'*s_n*q_n+repmat(((zz-1)*s_n+1:(zz-1)*s_n+s_n)',b_n,1))=[];
                    obj(constr_q)=-10000000001;
                    
                    [~,max_coordinate]=max(obj);
                    b_coord=floor((max_coordinate-1)/(q_n*s_n))+1;
                    q_coord=rem(floor((max_coordinate-1)/(s_n)),q_n)+1;
                    bprime_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=b_grid(b_coord);
                    qprime_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=q_grid(q_coord);
                    
                    V_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=obj(max_coordinate);
            
                end
           end
           
       end
      
       
       V(:,jj)=V_n(:,jj);
       bprime(:,jj)=bprime_n(:,jj);
       qprime(:,jj)=qprime_n(:,jj);
       
       part_help=find(V_p(:,jj)>V_n(:,jj));
       
       V(part_help,jj)=V_p(part_help,jj);
       bprime(part_help,jj)=bprime_p(part_help,jj);
       qprime(part_help,jj)=qprime_p(part_help,jj);
       sprime(part_help,jj)=sprime_p(part_help,jj);
       part(part_help,jj)=1;
       
       Time_since_start=duration(0,0,toc)
end        
             
        
        
    % Working Life
    
for jj=J_ret:-1:1
   jj
     
       for rr=1:nRq
           
           % Rqprime low
           
           Vprime_Rqprimelow=reshape(V(1:x_n*z_n,jj+1),z_n,x_n)';
           
           Vprime_p_Rqprimelow=interp2_lin(xprime_p_Rqprimelow,zprime_Rqprimelow,xprime_p_Rqprimelow_coord,zprime_Rqprimelow_coord,x_grid,z_grid,Vprime_Rqprimelow);
           Vprime_p_Rqprimelow=reshape(Vprime_p_Rqprimelow,b_n*q_n*s_n,nN*nU*nRs);
           Vprime_p_RqprimelowGN=[Vprime_p_Rqprimelow(:,1:nU*nRs)*(G*N(1))^(1-gamma) Vprime_p_Rqprimelow(:,nU*nRs+1:end)*(G*N(2))^(1-gamma)];
           EtVprime_p_Rqprimelow=Vprime_p_RqprimelowGN*prob_bqsind;
           clearvars Vprime_p_Rqprimelow Vprime_p_RqprimelowGN
           
           Vprime_n_Rqprimelow=interp2_lin(xprime_n_Rqprimelow,zprime_Rqprimelow,xprime_n_Rqprimelow_coord,zprime_Rqprimelow_coord,x_grid,z_grid,Vprime_Rqprimelow);
           Vprime_n_Rqprimelow=reshape(Vprime_n_Rqprimelow,b_n*q_n*s_n,nN*nU*nRs);
           Vprime_n_RqprimelowGN=[Vprime_n_Rqprimelow(:,1:nU*nRs)*(G*N(1))^(1-gamma) Vprime_n_Rqprimelow(:,nU*nRs+1:end)*(G*N(2))^(1-gamma)];
           EtVprime_n_Rqprimelow=Vprime_n_RqprimelowGN*prob_bqsind;
           clearvars Vprime_n_Rqprimelow Vprime_n_RqprimelowGN Vprime_Rqprimelow
           
           
           % Rqprime high 
            
           Vprime_Rqprimehigh=reshape(V(x_n*z_n+1:end,jj+1),z_n,x_n)';
           
           Vprime_p_Rqprimehigh=interp2_lin(xprime_p_Rqprimehigh,zprime_Rqprimehigh,xprime_p_Rqprimehigh_coord,zprime_Rqprimehigh_coord,x_grid,z_grid,Vprime_Rqprimehigh);
           Vprime_p_Rqprimehigh=reshape(Vprime_p_Rqprimehigh,b_n*q_n*s_n,nN*nU*nRs);
           Vprime_p_RqprimehighGN=[Vprime_p_Rqprimehigh(:,1:nU*nRs)*(G*N(1))^(1-gamma) Vprime_p_Rqprimehigh(:,nU*nRs+1:end)*(G*N(2))^(1-gamma)];
           EtVprime_p_Rqprimehigh=Vprime_p_RqprimehighGN*prob_bqsind;
           clearvars Vprime_p_Rqprimehigh Vprime_p_RqprimehighGN
           
           Vprime_n_Rqprimehigh=interp2_lin(xprime_n_Rqprimehigh,zprime_Rqprimehigh,xprime_n_Rqprimehigh_coord,zprime_Rqprimehigh_coord,x_grid,z_grid,Vprime_Rqprimehigh);
           Vprime_n_Rqprimehigh=reshape(Vprime_n_Rqprimehigh,b_n*q_n*s_n,nN*nU*nRs);   
           Vprime_n_RqprimehighGN=[Vprime_n_Rqprimehigh(:,1:nU*nRs)*(G*N(1))^(1-gamma) Vprime_n_Rqprimehigh(:,nU*nRs+1:end)*(G*N(2))^(1-gamma)];
           EtVprime_n_Rqprimehigh=Vprime_n_RqprimehighGN*prob_bqsind;
           clearvars Vprime_n_Rqprimehigh Vprime_n_RqprimehighGN Vprime_Rqprimehigh  
           
           
           EtVprime_p_bqsvec=ProbRq(rr,1).*EtVprime_p_Rqprimelow+ProbRq(rr,2).*EtVprime_p_Rqprimehigh;
           EtVprime_n_bqsvec=ProbRq(rr,1).*EtVprime_n_Rqprimelow+ProbRq(rr,2).*EtVprime_n_Rqprimehigh;
           
           for xx=1:x_n
                for zz=1:z_n
                    
                    %participating
                    c=x_grid(xx)+z_grid(zz)-phi-(bprime_grid_bqsvec+qprime_grid_bqsvec+sprime_grid_bqsvec*(1+psi_s));
                    constr_c=find(c<=0);
                    obj=util(c)+beta*EtVprime_p_bqsvec;
                    obj(constr_c)=-10000000000;
                    constr_q=(repelem(0:b_n-1,(zz-1)*s_n))'*s_n*q_n+repmat((1:(zz-1)*s_n)',b_n,1);
                    obj(constr_q)=-10000000001;
                    
                    [~,max_coordinate]=max(obj);
                    b_coord=floor((max_coordinate-1)/(q_n*s_n))+1;
                    q_coord=rem(floor((max_coordinate-1)/(s_n)),q_n)+1;
                    s_coord=rem(max_coordinate-1,s_n)+1;
                    bprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=b_grid(b_coord);
                    qprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=q_grid(q_coord);
                    sprime_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=s_grid(s_coord);
                    
                    V_p((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=obj(max_coordinate);
                   
                          
                    %not participating
                    c=x_grid(xx)-bprime_grid_bqsvec;
                    constr_c=find(c<=0);
                    obj=util(c)+beta*EtVprime_n_bqsvec;
                    obj(constr_c)=-10000000000;
                    constr_q=(1:b_n*q_n*s_n)';
                    constr_q((repelem(0:b_n-1,s_n))'*s_n*q_n+repmat(((zz-1)*s_n+1:(zz-1)*s_n+s_n)',b_n,1))=[];
                    obj(constr_q)=-10000000001;
                    
                    [~,max_coordinate]=max(obj);
                    b_coord=floor((max_coordinate-1)/(q_n*s_n))+1;
                    q_coord=rem(floor((max_coordinate-1)/(s_n)),q_n)+1;
                    bprime_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=b_grid(b_coord);
                    qprime_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=q_grid(q_coord);
                    
                    V_n((rr-1)*x_n*z_n+(xx-1)*z_n+zz,jj)=obj(max_coordinate);
            
                end
           end
       end
       
       V(:,jj)=V_n(:,jj);
       bprime(:,jj)=bprime_n(:,jj);
       qprime(:,jj)=qprime_n(:,jj);
       
       part_help=find(V_p(:,jj)>V_n(:,jj));
       
       V(part_help,jj)=V_p(part_help,jj);
       bprime(part_help,jj)=bprime_p(part_help,jj);
       qprime(part_help,jj)=qprime_p(part_help,jj);
       sprime(part_help,jj)=sprime_p(part_help,jj);
       part(part_help,jj)=1;
       
end

clearvars -except delta beta gamma Rb sigmaU sigmaN G G_ret Rqexcess phi psi_s J J_ret ret ...
    nU ProbU U nN ProbN N nRq ProbRq Rq nRs ProbRs Rs ...
    b_min b_max b_n b_grid q_min q_max q_n q_grid s_min s_max s_n s_grid x_min x_max x_n x_grid z_min z_max z_n z_grid ...
    V_p bprime_p qprime_p sprime_p V_n bprime_n qprime_n V bprime qprime sprime part

save Model_1_PolFun.mat


    